import pytest
import salt.utils.data
import salt.utils.reactor as reactor
from tests.support.mock import MagicMock, call, patch


@pytest.fixture
def minion_config():
    opts = salt.config.DEFAULT_MINION_OPTS.copy()
    opts["__role"] = "minion"
    return opts


@pytest.fixture
def master_config():
    opts = salt.config.DEFAULT_MASTER_OPTS.copy()
    opts["__role"] = "master"
    return opts


@pytest.fixture
def minion_reactor(minion_config):
    return reactor.Reactor(minion_config)


@pytest.fixture
def master_reactor(master_config):
    return reactor.Reactor(master_config)


@pytest.mark.skip_on_windows(reason="Reactors unavailable on Windows")
def test_run_minion_reactor(minion_config, minion_reactor):
    """
    Ensure that list_reactors() returns the correct list of reactor SLS
    files for each tag.
    """

    with patch.object(salt.utils.event, "get_event", MagicMock()):
        os_nice_mock = MagicMock(return_value=True)
        with patch("os.nice", os_nice_mock):
            # By default os.nice should not be called
            minion_reactor.run()
            assert os_nice_mock.mock_calls == []

            # if reactor_niceness is set, os.nice should be called with that value
            with patch.dict(minion_config, {"reactor_niceness": 9}):
                minion_reactor.run()
                calls = [call(9)]
                os_nice_mock.assert_has_calls(calls)


@pytest.mark.skip_on_windows(reason="Reactors unavailable on Windows")
def test_run_master_reactor(master_config, master_reactor):
    """
    Ensure that list_reactors() returns the correct list of reactor SLS
    files for each tag.
    """

    with patch.object(salt.utils.event, "get_event", MagicMock()):
        os_nice_mock = MagicMock(return_value=True)
        with patch("os.nice", os_nice_mock):
            # By default os.nice should not be called
            master_reactor.run()
            assert os_nice_mock.mock_calls == []

            # if reactor_niceness is set, os.nice should be called with that value
            with patch.dict(master_config, {"reactor_niceness": 9}):
                master_reactor.run()
                calls = [call(9)]
                os_nice_mock.assert_has_calls(calls)



"""
Tests for salt.modules.zpool

:codeauthor:    Nitin Madhok <nmadhok@g.clemson.edu>, Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs
:platform:      illumos,freebsd,linux
"""

import pytest
import salt.loader
import salt.modules.zpool as zpool
import salt.utils.decorators
import salt.utils.decorators.path
import salt.utils.zfs
from salt.utils.odict import OrderedDict
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf
from tests.support.zfs import ZFSMockData


class ZpoolTestCase(TestCase, LoaderModuleMockMixin):
    """
    This class contains a set of functions that test salt.modules.zpool module
    """

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        self.utils_patch = ZFSMockData().get_patched_utils()
        for key in ("opts", "utils_patch"):
            self.addCleanup(delattr, self, key)
        utils = salt.loader.utils(
            opts, whitelist=["zfs", "args", "systemd", "path", "platform"]
        )
        zpool_obj = {zpool: {"__opts__": opts, "__utils__": utils}}

        return zpool_obj

    @pytest.mark.slow_test
    def test_exists_success(self):
        """
        Tests successful return of exists function
        """
        ret = {}
        ret["stdout"] = (
            "NAME      SIZE  ALLOC   FREE    CAP  DEDUP  HEALTH  ALTROOT\n"
            "myzpool   149G   128K   149G     0%  1.00x  ONLINE  -"
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertTrue(zpool.exists("myzpool"))

    @pytest.mark.slow_test
    def test_exists_failure(self):
        """
        Tests failure return of exists function
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertFalse(zpool.exists("myzpool"))

    def test_healthy(self):
        """
        Tests successful return of healthy function
        """
        ret = {}
        ret["stdout"] = "all pools are healthy"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertTrue(zpool.healthy())

    def test_status(self):
        """
        Tests successful return of status function
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "  pool: mypool",
                " state: ONLINE",
                "  scan: scrub repaired 0 in 0h6m with 0 errors on Mon Dec 21 02:06:17"
                " 2015",
                "config:",
                "",
                "\tNAME        STATE     READ WRITE CKSUM",
                "\tmypool      ONLINE       0     0     0",
                "\t  mirror-0  ONLINE       0     0     0",
                "\t    c2t0d0  ONLINE       0     0     0",
                "\t    c2t1d0  ONLINE       0     0     0",
                "",
                "errors: No known data errors",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.status()
            self.assertEqual("ONLINE", ret["mypool"]["state"])

    def test_status_with_colons_in_vdevs(self):
        """
        Tests successful return of status function
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "  pool: mypool",
                " state: ONLINE",
                "  scan: scrub repaired 0 in 0h6m with 0 errors on Mon Dec 21 02:06:17"
                " 2015",
                "config:",
                "",
                "\tNAME        STATE     READ WRITE CKSUM",
                "\tmypool      ONLINE       0     0     0",
                "\t  mirror-0  ONLINE       0     0     0",
                "\t    usb-WD_My_Book_Duo_25F6_....32-0:0  ONLINE       0     0     0",
                "\t    usb-WD_My_Book_Duo_25F6_....32-0:1  ONLINE       0     0     0",
                "",
                "errors: No known data errors",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.status()
            self.assertEqual("ONLINE", ret["mypool"]["state"])

    @pytest.mark.slow_test
    def test_iostat(self):
        """
        Tests successful return of iostat function
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "               capacity     operations    bandwidth",
                "pool        alloc   free   read  write   read  write",
                "----------  -----  -----  -----  -----  -----  -----",
                "mypool      46.7G  64.3G      4     19   113K   331K",
                "  mirror    46.7G  64.3G      4     19   113K   331K",
                "    c2t0d0      -      -      1     10   114K   334K",
                "    c2t1d0      -      -      1     10   114K   334K",
                "----------  -----  -----  -----  -----  -----  -----",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.iostat("mypool", parsable=False)
            self.assertEqual("46.7G", ret["mypool"]["capacity-alloc"])

    def test_iostat_parsable(self):
        """
        Tests successful return of iostat function

        .. note:
            The command output is the same as the non parsable!
            There is no -p flag for zpool iostat, but our type
            conversions can handle this!
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "               capacity     operations    bandwidth",
                "pool        alloc   free   read  write   read  write",
                "----------  -----  -----  -----  -----  -----  -----",
                "mypool      46.7G  64.3G      4     19   113K   331K",
                "  mirror    46.7G  64.3G      4     19   113K   331K",
                "    c2t0d0      -      -      1     10   114K   334K",
                "    c2t1d0      -      -      1     10   114K   334K",
                "----------  -----  -----  -----  -----  -----  -----",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.iostat("mypool", parsable=True)
            self.assertEqual(50143743180, ret["mypool"]["capacity-alloc"])

    def test_list(self):
        """
        Tests successful return of list function
        """
        ret = {}
        ret["stdout"] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.list_(parsable=False)
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                ("size", "1.81T"),
                                ("alloc", "661G"),
                                ("free", "1.17T"),
                                ("cap", "35%"),
                                ("frag", "11%"),
                                ("health", "ONLINE"),
                            ]
                        ),
                    )
                ]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_list_parsable(self):
        """
        Tests successful return of list function with parsable output
        """
        ret = {}
        ret["stdout"] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.list_(parsable=True)
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                ("size", 1990116046274),
                                ("alloc", 709743345664),
                                ("free", 1286428604497),
                                ("cap", "35%"),
                                ("frag", "11%"),
                                ("health", "ONLINE"),
                            ]
                        ),
                    )
                ]
            )
            self.assertEqual(ret, res)

    def test_get(self):
        """
        Tests successful return of get function
        """
        ret = {}
        ret["stdout"] = "mypool\tsize\t1.81T\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "size", parsable=False)
            res = OrderedDict(OrderedDict([("size", "1.81T")]))
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_get_parsable(self):
        """
        Tests successful return of get function with parsable output
        """
        ret = {}
        ret["stdout"] = "mypool\tsize\t1.81T\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "size", parsable=True)
            res = OrderedDict(OrderedDict([("size", 1990116046274)]))
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_get_whitespace(self):
        """
        Tests successful return of get function with a string with whitespaces
        """
        ret = {}
        ret["stdout"] = "mypool\tcomment\tmy testing pool\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "comment")
            res = OrderedDict(OrderedDict([("comment", "my testing pool")]))
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_scrub_start(self):
        """
        Tests start of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool")
            res = OrderedDict(OrderedDict([("scrubbing", True)]))
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_scrub_pause(self):
        """
        Tests pause of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool", pause=True)
            res = OrderedDict(OrderedDict([("scrubbing", False)]))
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_scrub_stop(self):
        """
        Tests pauze of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool", stop=True)
            res = OrderedDict(OrderedDict([("scrubbing", False)]))
            self.assertEqual(ret, res)

    def test_split_success(self):
        """
        Tests split on success
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict([("split", True)])
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_split_exist_new(self):
        """
        Tests split on exising new pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "Unable to split datapool: pool already exists"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [
                    ("split", False),
                    ("error", "Unable to split datapool: pool already exists"),
                ]
            )
            self.assertEqual(ret, res)

    def test_split_missing_pool(self):
        """
        Tests split on missing source pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'datapool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [("split", False), ("error", "cannot open 'datapool': no such pool")]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_split_not_mirror(self):
        """
        Tests split on source pool is not a mirror
        """
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "Unable to split datapool: Source pool must be composed only of mirrors"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [
                    ("split", False),
                    (
                        "error",
                        "Unable to split datapool: Source pool must be composed only of"
                        " mirrors",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_success(self):
        """
        Tests labelclear on successful label removal
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict([("labelcleared", True)])
            self.assertEqual(ret, res)

    def test_labelclear_nodevice(self):
        """
        Tests labelclear on non existing device
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "failed to open /dev/rdsk/c0t0d0: No such file or directory"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    (
                        "error",
                        "failed to open /dev/rdsk/c0t0d0: No such file or directory",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_cleared(self):
        """
        Tests labelclear on device with no label
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "failed to read label from /dev/rdsk/c0t0d0"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    ("error", "failed to read label from /dev/rdsk/c0t0d0"),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_exported(self):
        """
        Tests labelclear on device with from exported pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "\n".join(
            [
                "use '-f' to override the following error:",
                '/dev/rdsk/c0t0d0 is a member of exported pool "mypool"',
            ]
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    (
                        "error",
                        "use 'force=True' to override the following"
                        " error:\n/dev/rdsk/c0t0d0 is a member of exported pool"
                        ' "mypool"',
                    ),
                ]
            )
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which("mkfile"), "Cannot find mkfile executable")
    def test_create_file_vdev_success(self):
        """
        Tests create_file_vdev when out of space
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.create_file_vdev("64M", "/vdisks/disk0")
            res = OrderedDict([("/vdisks/disk0", "created")])
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which("mkfile"), "Cannot find mkfile executable")
    def test_create_file_vdev_nospace(self):
        """
        Tests create_file_vdev when out of space
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = (
            "/vdisks/disk0: initialized 10424320 of 67108864 bytes: No space left on"
            " device"
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.create_file_vdev("64M", "/vdisks/disk0")
            res = OrderedDict(
                [
                    ("/vdisks/disk0", "failed"),
                    (
                        "error",
                        OrderedDict(
                            [
                                (
                                    "/vdisks/disk0",
                                    " initialized 10424320 of 67108864 bytes: No space"
                                    " left on device",
                                ),
                            ]
                        ),
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_export_success(self):
        """
        Tests export
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.export("mypool")
            res = OrderedDict([("exported", True)])
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_export_nopool(self):
        """
        Tests export when the pool does not exists
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.export("mypool")
            res = OrderedDict(
                [("exported", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_import_success(self):
        """
        Tests import
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict([("imported", True)])
            self.assertEqual(ret, res)

    def test_import_duplicate(self):
        """
        Tests import with already imported pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "\n".join(
            [
                "cannot import 'mypool': a pool with that name already exists",
                "use the form 'zpool import <pool | id> <newpool>' to give it a new"
                " name",
            ]
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict(
                [
                    ("imported", False),
                    (
                        "error",
                        "cannot import 'mypool': a pool with that name already"
                        " exists\nuse the form 'zpool import <pool | id> <newpool>' to"
                        " give it a new name",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_import_nopool(self):
        """
        Tests import
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot import 'mypool': no such pool available"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict(
                [
                    ("imported", False),
                    ("error", "cannot import 'mypool': no such pool available"),
                ]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_online_success(self):
        """
        Tests online
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.online("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict([("onlined", True)])
            self.assertEqual(ret, res)

    def test_online_nodevice(self):
        """
        Tests online
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot online /dev/rdsk/c0t0d1: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.online("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("onlined", False),
                    ("error", "cannot online /dev/rdsk/c0t0d1: no such device in pool"),
                ]
            )
            self.assertEqual(ret, res)

    def test_offline_success(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict([("offlined", True)])
            self.assertEqual(ret, res)

    def test_offline_nodevice(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot offline /dev/rdsk/c0t0d1: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("offlined", False),
                    (
                        "error",
                        "cannot offline /dev/rdsk/c0t0d1: no such device in pool",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_offline_noreplica(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot offline /dev/rdsk/c0t0d1: no valid replicas"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("offlined", False),
                    ("error", "cannot offline /dev/rdsk/c0t0d1: no valid replicas"),
                ]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_reguid_success(self):
        """
        Tests reguid
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reguid("mypool")
            res = OrderedDict([("reguided", True)])
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_reguid_nopool(self):
        """
        Tests reguid with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reguid("mypool")
            res = OrderedDict(
                [("reguided", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_reopen_success(self):
        """
        Tests reopen
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reopen("mypool")
            res = OrderedDict([("reopened", True)])
            self.assertEqual(ret, res)

    def test_reopen_nopool(self):
        """
        Tests reopen with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reopen("mypool")
            res = OrderedDict(
                [("reopened", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    def test_upgrade_success(self):
        """
        Tests upgrade
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.upgrade("mypool")
            res = OrderedDict([("upgraded", True)])
            self.assertEqual(ret, res)

    def test_upgrade_nopool(self):
        """
        Tests upgrade with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.upgrade("mypool")
            res = OrderedDict(
                [("upgraded", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @pytest.mark.slow_test
    def test_history_success(self):
        """
        Tests history
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "History for 'mypool':",
                "2018-01-18.16:56:12 zpool create -f mypool /dev/rdsk/c0t0d0",
                "2018-01-19.16:01:55 zpool attach -f mypool /dev/rdsk/c0t0d0"
                " /dev/rdsk/c0t0d1",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.history("mypool")
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                (
                                    "2018-01-18.16:56:12",
                                    "zpool create -f mypool /dev/rdsk/c0t0d0",
                                ),
                                (
                                    "2018-01-19.16:01:55",
                                    "zpool attach -f mypool /dev/rdsk/c0t0d0"
                                    " /dev/rdsk/c0t0d1",
                                ),
                            ]
                        ),
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_history_nopool(self):
        """
        Tests history with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.history("mypool")
            res = OrderedDict([("error", "cannot open 'mypool': no such pool")])
            self.assertEqual(ret, res)

    def test_clear_success(self):
        """
        Tests clear
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool")
            res = OrderedDict([("cleared", True)])
            self.assertEqual(ret, res)

    def test_clear_nopool(self):
        """
        Tests clear with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool")
            res = OrderedDict(
                [("cleared", False), ("error", "cannot open 'mypool': no such pool")]
            )

    def test_clear_nodevice(self):
        """
        Tests clear with non existign device
        """
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "cannot clear errors for /dev/rdsk/c0t0d0: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict(
                [
                    ("cleared", False),
                    (
                        "error",
                        "cannot clear errors for /dev/rdsk/c0t0d0: no such device in"
                        " pool",
                    ),
                ]
            )
            self.assertEqual(ret, res)



from salt.runners import manage
from tests.support.unit import TestCase


class ManageTest(TestCase):
    def test_deprecation_58638(self):
        # check that type error will be raised
        self.assertRaises(TypeError, manage.list_state, show_ipv4="data")

        # check that show_ipv4 will raise an error
        try:
            manage.list_state(  # pylint: disable=unexpected-keyword-arg
                show_ipv4="data"
            )
        except TypeError as no_show_ipv4:
            self.assertEqual(
                str(no_show_ipv4),
                "list_state() got an unexpected keyword argument 'show_ipv4'",
            )



"""
Functions for analyzing/parsing docstrings
"""
import logging
import re

log = logging.getLogger(__name__)


def strip_rst(docs):
    """
    Strip/replace reStructuredText directives in docstrings
    """
    for func, docstring in docs.items():
        log.debug("Stripping docstring for %s", func)
        if not docstring:
            continue
        docstring_new = docstring
        for regex, repl in (
            (r" *.. code-block:: \S+\n{1,2}", ""),
            (".. note::", "Note:"),
            (".. warning::", "Warning:"),
            (".. versionadded::", "New in version"),
            (".. versionchanged::", "Changed in version"),
        ):
            try:
                docstring_new = re.sub(regex, repl, docstring_new)
            except Exception:  # pylint: disable=broad-except
                log.debug(
                    "Exception encountered while matching regex %r to "
                    "docstring for function %s",
                    regex,
                    func,
                    exc_info=True,
                )
        if docstring != docstring_new:
            docs[func] = docstring_new
    return docs


def parse_docstring(docstring):
    """
    Parse a docstring into its parts.

    Currently only parses dependencies, can be extended to parse whatever is
    needed.

    Parses into a dictionary:
        {
            'full': full docstring,
            'deps': list of dependencies (empty list if none)
        }
    """
    # First try with regex search for :depends:
    ret = {"full": docstring}
    regex = r"([ \t]*):depends:[ \t]+- (\w+)[^\n]*\n(\1[ \t]+- (\w+)[^\n]*\n)*"
    match = re.search(regex, docstring, re.M)
    if match:
        deps = []
        regex = r"- (\w+)"
        for line in match.group(0).strip().splitlines():
            deps.append(re.search(regex, line).group(1))
        ret["deps"] = deps
        return ret
    # Try searching for a one-liner instead
    else:
        txt = "Required python modules: "
        data = docstring.splitlines()
        dep_list = list(x for x in data if x.strip().startswith(txt))
        if not dep_list:
            ret["deps"] = []
            return ret
        deps = dep_list[0].replace(txt, "").strip().split(", ")
        ret["deps"] = deps
        return ret



"""
Simple grain to merge the opts into the grains directly if the grain_opts
configuration value is set.
"""


def opts():
    """
    Return the minion configuration settings
    """
    if __opts__.get("grain_opts", False) or (
        isinstance(__pillar__, dict) and __pillar__.get("grain_opts", False)
    ):
        return __opts__
    return {}



import logging
import os
import shutil
import subprocess

import pytest
import salt.utils.platform

log = logging.getLogger(__name__)


@pytest.fixture(scope="package", autouse=True)
def skip_on_tcp_transport(request):
    if request.config.getoption("--transport") == "tcp":
        pytest.skip("Multimaster under the TPC transport is not working. See #59053")


@pytest.fixture(scope="package")
def salt_mm_master_1(request, salt_factories):
    config_defaults = {
        "open_mode": True,
        "transport": request.config.getoption("--transport"),
    }
    config_overrides = {
        "interface": "127.0.0.1",
    }

    factory = salt_factories.salt_master_daemon(
        "mm-master-1",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=debug"],
    )
    with factory.started(start_timeout=120):
        yield factory


@pytest.fixture(scope="package")
def mm_master_1_salt_cli(salt_mm_master_1):
    return salt_mm_master_1.salt_cli(timeout=120)


@pytest.fixture(scope="package")
def salt_mm_master_2(salt_factories, salt_mm_master_1):
    if salt.utils.platform.is_darwin() or salt.utils.platform.is_freebsd():
        subprocess.check_output(["ifconfig", "lo0", "alias", "127.0.0.2", "up"])

    config_defaults = {
        "open_mode": True,
        "transport": salt_mm_master_1.config["transport"],
    }
    config_overrides = {
        "interface": "127.0.0.2",
    }

    # Use the same ports for both masters, they are binding to different interfaces
    for key in (
        "ret_port",
        "publish_port",
    ):
        config_overrides[key] = salt_mm_master_1.config[key]
    factory = salt_factories.salt_master_daemon(
        "mm-master-2",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=debug"],
    )

    # The secondary salt master depends on the primarily salt master fixture
    # because we need to clone the keys
    for keyfile in ("master.pem", "master.pub"):
        shutil.copyfile(
            os.path.join(salt_mm_master_1.config["pki_dir"], keyfile),
            os.path.join(factory.config["pki_dir"], keyfile),
        )
    with factory.started(start_timeout=120):
        yield factory


@pytest.fixture(scope="package")
def mm_master_2_salt_cli(salt_mm_master_2):
    return salt_mm_master_2.salt_cli(timeout=120)


@pytest.fixture(scope="package")
def salt_mm_minion_1(salt_mm_master_1, salt_mm_master_2):
    config_defaults = {
        "transport": salt_mm_master_1.config["transport"],
    }

    mm_master_1_port = salt_mm_master_1.config["ret_port"]
    mm_master_1_addr = salt_mm_master_1.config["interface"]
    mm_master_2_port = salt_mm_master_2.config["ret_port"]
    mm_master_2_addr = salt_mm_master_2.config["interface"]
    config_overrides = {
        "master": [
            "{}:{}".format(mm_master_1_addr, mm_master_1_port),
            "{}:{}".format(mm_master_2_addr, mm_master_2_port),
        ],
        "test.foo": "baz",
    }
    factory = salt_mm_master_1.salt_minion_daemon(
        "mm-minion-1",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=debug"],
    )
    with factory.started(start_timeout=120):
        yield factory


@pytest.fixture(scope="package")
def salt_mm_minion_2(salt_mm_master_1, salt_mm_master_2):
    config_defaults = {
        "transport": salt_mm_master_1.config["transport"],
    }

    mm_master_1_port = salt_mm_master_1.config["ret_port"]
    mm_master_1_addr = salt_mm_master_1.config["interface"]
    mm_master_2_port = salt_mm_master_2.config["ret_port"]
    mm_master_2_addr = salt_mm_master_2.config["interface"]
    config_overrides = {
        "master": [
            "{}:{}".format(mm_master_1_addr, mm_master_1_port),
            "{}:{}".format(mm_master_2_addr, mm_master_2_port),
        ],
        "test.foo": "baz",
    }
    factory = salt_mm_master_2.salt_minion_daemon(
        "mm-minion-2",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=debug"],
    )
    with factory.started(start_timeout=120):
        yield factory



"""
    :codeauthor: Erik Johnson <erik@saltstack.com>
"""

import textwrap

import pytest
import salt.utils.platform
import salt.utils.stringutils
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import DEFAULT, MagicMock, mock_open, patch
from tests.support.unit import TestCase, skipIf

try:
    import spwd
except ImportError:
    pass

try:
    import salt.modules.linux_shadow as shadow

    HAS_SHADOW = True
except ImportError:
    HAS_SHADOW = False

_PASSWORD = "lamepassword"

# Not testing blowfish as it is not available on most Linux distros
_HASHES = dict(
    md5=dict(pw_salt="TgIp9OTu", pw_hash="$1$TgIp9OTu$.d0FFP6jVi5ANoQmk6GpM1"),
    sha256=dict(
        pw_salt="3vINbSrC",
        pw_hash="$5$3vINbSrC$hH8A04jAY3bG123yU4FQ0wvP678QDTvWBhHHFbz6j0D",
    ),
    sha512=dict(
        pw_salt="PiGA3V2o",
        pw_hash="$6$PiGA3V2o$/PrntRYufz49bRV/V5Eb1V6DdHaS65LB0fu73Tp/xxmDFr6HWJKptY2TvHRDViXZugWpnAcOnrbORpOgZUGTn.",
    ),
)


@skipIf(not salt.utils.platform.is_linux(), "minion is not Linux")
@skipIf(not HAS_SHADOW, "shadow module is not available")
class LinuxShadowTest(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {shadow: {}}

    def test_gen_password(self):
        """
        Test shadow.gen_password
        """
        self.assertTrue(HAS_SHADOW)
        for algorithm, hash_info in _HASHES.items():
            self.assertEqual(
                shadow.gen_password(
                    _PASSWORD, crypt_salt=hash_info["pw_salt"], algorithm=algorithm
                ),
                hash_info["pw_hash"],
            )

    def test_set_password(self):
        """
        Test the corner case in which shadow.set_password is called for a user
        that has an entry in /etc/passwd but not /etc/shadow.
        """
        data = {
            "/etc/shadow": salt.utils.stringutils.to_bytes(
                textwrap.dedent(
                    """\
                foo:orighash:17955::::::
                bar:somehash:17955::::::
                """
                )
            ),
            "*": Exception("Attempted to open something other than /etc/shadow"),
        }
        isfile_mock = MagicMock(
            side_effect=lambda x: True if x == "/etc/shadow" else DEFAULT
        )
        password = "newhash"
        shadow_info_mock = MagicMock(return_value={"passwd": password})

        #
        # CASE 1: Normal password change
        #
        user = "bar"
        user_exists_mock = MagicMock(
            side_effect=lambda x, **y: 0 if x == ["id", user] else DEFAULT
        )
        with patch(
            "salt.utils.files.fopen", mock_open(read_data=data)
        ) as shadow_mock, patch("os.path.isfile", isfile_mock), patch.object(
            shadow, "info", shadow_info_mock
        ), patch.dict(
            shadow.__salt__, {"cmd.retcode": user_exists_mock}
        ), patch.dict(
            shadow.__grains__, {"os": "CentOS"}
        ):
            result = shadow.set_password(user, password, use_usermod=False)

        assert result
        filehandles = shadow_mock.filehandles["/etc/shadow"]
        # We should only have opened twice, once to read the contents and once
        # to write.
        assert len(filehandles) == 2
        # We're rewriting the entire file
        assert filehandles[1].mode == "w+"
        # We should be calling writelines instead of write, to rewrite the
        # entire file.
        assert len(filehandles[1].writelines_calls) == 1
        # Make sure we wrote the correct info
        lines = filehandles[1].writelines_calls[0]
        # Should only have the same two users in the file
        assert len(lines) == 2
        # The first line should be unchanged
        assert lines[0] == "foo:orighash:17955::::::\n"
        # The second line should have the new password hash
        assert lines[1].split(":")[:2] == [user, password]

        #
        # CASE 2: Corner case: no /etc/shadow entry for user
        #
        user = "baz"
        user_exists_mock = MagicMock(
            side_effect=lambda x, **y: 0 if x == ["id", user] else DEFAULT
        )
        with patch(
            "salt.utils.files.fopen", mock_open(read_data=data)
        ) as shadow_mock, patch("os.path.isfile", isfile_mock), patch.object(
            shadow, "info", shadow_info_mock
        ), patch.dict(
            shadow.__salt__, {"cmd.retcode": user_exists_mock}
        ), patch.dict(
            shadow.__grains__, {"os": "CentOS"}
        ):
            result = shadow.set_password(user, password, use_usermod=False)

        assert result
        filehandles = shadow_mock.filehandles["/etc/shadow"]
        # We should only have opened twice, once to read the contents and once
        # to write.
        assert len(filehandles) == 2
        # We're just appending to the file, not rewriting
        assert filehandles[1].mode == "a+"
        # We should only have written to the file once
        assert len(filehandles[1].write_calls) == 1
        # Make sure we wrote the correct info
        assert filehandles[1].write_calls[0].split(":")[:2] == [user, password]

    def test_info(self):
        """
        Test if info shows the correct user information
        """

        # First test is with a succesful call
        expected_result = [
            ("expire", -1),
            ("inact", -1),
            ("lstchg", 31337),
            ("max", 99999),
            ("min", 0),
            ("name", "foo"),
            ("passwd", _HASHES["sha512"]["pw_hash"]),
            ("warn", 7),
        ]
        getspnam_return = spwd.struct_spwd(
            ["foo", _HASHES["sha512"]["pw_hash"], 31337, 0, 99999, 7, -1, -1, -1]
        )
        with patch("spwd.getspnam", return_value=getspnam_return):
            result = shadow.info("foo")
            self.assertEqual(
                expected_result, sorted(result.items(), key=lambda x: x[0])
            )

        # The next two is for a non-existent user
        expected_result = [
            ("expire", ""),
            ("inact", ""),
            ("lstchg", ""),
            ("max", ""),
            ("min", ""),
            ("name", ""),
            ("passwd", ""),
            ("warn", ""),
        ]
        # We get KeyError exception for non-existent users in glibc based systems
        getspnam_return = KeyError
        with patch("spwd.getspnam", side_effect=getspnam_return):
            result = shadow.info("foo")
            self.assertEqual(
                expected_result, sorted(result.items(), key=lambda x: x[0])
            )
        # And FileNotFoundError in musl based systems
        getspnam_return = FileNotFoundError
        with patch("spwd.getspnam", side_effect=getspnam_return):
            result = shadow.info("foo")
            self.assertEqual(
                expected_result, sorted(result.items(), key=lambda x: x[0])
            )

    @pytest.mark.skip_if_not_root
    def test_list_users(self):
        """
        Test if it returns a list of all users
        """
        self.assertTrue(shadow.list_users())



"""
Publish a command from a minion to a target
"""

import logging
import time

import salt.crypt
import salt.payload
import salt.transport.client
import salt.utils.args
from salt.exceptions import SaltInvocationError, SaltReqTimeoutError

log = logging.getLogger(__name__)

__virtualname__ = "publish"


def __virtual__():
    return (
        __virtualname__ if __opts__.get("transport", "") in ("zeromq", "tcp") else False
    )


def _parse_args(arg):
    """
    yamlify `arg` and ensure its outermost datatype is a list
    """
    yaml_args = salt.utils.args.yamlify_arg(arg)

    if yaml_args is None:
        return []
    elif not isinstance(yaml_args, list):
        return [yaml_args]
    else:
        return yaml_args


def _publish(
    tgt,
    fun,
    arg=None,
    tgt_type="glob",
    returner="",
    timeout=5,
    form="clean",
    wait=False,
    via_master=None,
):
    """
    Publish a command from the minion out to other minions, publications need
    to be enabled on the Salt master and the minion needs to have permission
    to publish the command. The Salt master will also prevent a recursive
    publication loop, this means that a minion cannot command another minion
    to command another minion as that would create an infinite command loop.

    The arguments sent to the minion publish function are separated with
    commas. This means that for a minion executing a command with multiple
    args it will look like this::

        salt system.example.com publish.publish '*' user.add 'foo,1020,1020'

    CLI Example:

    .. code-block:: bash

        salt system.example.com publish.publish '*' cmd.run 'ls -la /tmp'
    """
    if "master_uri" not in __opts__:
        log.error(
            "Cannot run publish commands without a connection to a salt master. No"
            " command sent."
        )
        return {}
    if fun.startswith("publish."):
        log.info("Cannot publish publish calls. Returning {}")
        return {}

    arg = _parse_args(arg)

    if via_master:
        if "master_uri_list" not in __opts__:
            raise SaltInvocationError(
                message=(
                    "Could not find list of masters in minion "
                    "configuration but `via_master` was specified."
                )
            )
        else:
            # Find the master in the list of master_uris generated by the minion base class
            matching_master_uris = [
                master
                for master in __opts__["master_uri_list"]
                if "//{}:".format(via_master) in master
            ]

            if not matching_master_uris:
                raise SaltInvocationError(
                    "Could not find match for {} in list of configured "
                    "masters {} when using `via_master` option".format(
                        via_master, __opts__["master_uri_list"]
                    )
                )

            if len(matching_master_uris) > 1:
                # If we have multiple matches, consider this a non-fatal error
                # and continue with whatever we found first.
                log.warning(
                    "The `via_master` flag found "
                    "more than one possible match found for %s when "
                    "evaluating list %s",
                    via_master,
                    __opts__["master_uri_list"],
                )
            master_uri = matching_master_uris.pop()
    else:
        # If no preference is expressed by the user, just publish to the first master
        # in the list.
        master_uri = __opts__["master_uri"]

    log.info("Publishing '%s' to %s", fun, master_uri)
    auth = salt.crypt.SAuth(__opts__)
    tok = auth.gen_token(b"salt")
    load = {
        "cmd": "minion_pub",
        "fun": fun,
        "arg": arg,
        "tgt": tgt,
        "tgt_type": tgt_type,
        "ret": returner,
        "tok": tok,
        "tmo": timeout,
        "form": form,
        "id": __opts__["id"],
        "no_parse": __opts__.get("no_parse", []),
    }

    with salt.transport.client.ReqChannel.factory(
        __opts__, master_uri=master_uri
    ) as channel:
        try:
            peer_data = channel.send(load)
        except SaltReqTimeoutError:
            return "'{}' publish timed out".format(fun)
        if not peer_data:
            return {}
        # CLI args are passed as strings, re-cast to keep time.sleep happy
        if wait:
            loop_interval = 0.3
            matched_minions = set(peer_data["minions"])
            returned_minions = set()
            loop_counter = 0
            while len(returned_minions ^ matched_minions) > 0:
                load = {
                    "cmd": "pub_ret",
                    "id": __opts__["id"],
                    "tok": tok,
                    "jid": peer_data["jid"],
                }
                ret = channel.send(load)
                returned_minions = set(ret.keys())

                end_loop = False
                if returned_minions >= matched_minions:
                    end_loop = True
                elif (loop_interval * loop_counter) > timeout:
                    # This may be unnecessary, but I am paranoid
                    if len(returned_minions) < 1:
                        return {}
                    end_loop = True

                if end_loop:
                    if form == "clean":
                        cret = {}
                        for host in ret:
                            cret[host] = ret[host]["ret"]
                        return cret
                    else:
                        return ret
                loop_counter = loop_counter + 1
                time.sleep(loop_interval)
        else:
            time.sleep(float(timeout))
            load = {
                "cmd": "pub_ret",
                "id": __opts__["id"],
                "tok": tok,
                "jid": peer_data["jid"],
            }
            ret = channel.send(load)
            if form == "clean":
                cret = {}
                for host in ret:
                    cret[host] = ret[host]["ret"]
                return cret
            else:
                return ret


def publish(
    tgt, fun, arg=None, tgt_type="glob", returner="", timeout=5, via_master=None
):
    """
    Publish a command from the minion out to other minions.

    Publications need to be enabled on the Salt master and the minion
    needs to have permission to publish the command. The Salt master
    will also prevent a recursive publication loop, this means that a
    minion cannot command another minion to command another minion as
    that would create an infinite command loop.

    The ``tgt_type`` argument is used to pass a target other than a glob into
    the execution, the available options are:

    - glob
    - pcre
    - grain
    - grain_pcre
    - pillar
    - pillar_pcre
    - ipcidr
    - range
    - compound

    .. versionchanged:: 2017.7.0
        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
        releases must use ``expr_form``.

    Note that for pillar matches must be exact, both in the pillar matcher
    and the compound matcher. No globbing is supported.

    The arguments sent to the minion publish function are separated with
    commas. This means that for a minion executing a command with multiple
    args it will look like this:

    .. code-block:: bash

        salt system.example.com publish.publish '*' user.add 'foo,1020,1020'
        salt system.example.com publish.publish 'os:Fedora' network.interfaces '' grain

    CLI Example:

    .. code-block:: bash

        salt system.example.com publish.publish '*' cmd.run 'ls -la /tmp'


    .. admonition:: Attention

        If you need to pass a value to a function argument and that value
        contains an equal sign, you **must** include the argument name.
        For example:

        .. code-block:: bash

            salt '*' publish.publish test.kwarg arg='cheese=spam'

        Multiple keyword arguments should be passed as a list.

        .. code-block:: bash

            salt '*' publish.publish test.kwarg arg="['cheese=spam','spam=cheese']"


    When running via salt-call, the `via_master` flag may be set to specific which
    master the publication should be sent to. Only one master may be specified. If
    unset, the publication will be sent only to the first master in minion configuration.
    """
    return _publish(
        tgt,
        fun,
        arg=arg,
        tgt_type=tgt_type,
        returner=returner,
        timeout=timeout,
        form="clean",
        wait=True,
        via_master=via_master,
    )


def full_data(tgt, fun, arg=None, tgt_type="glob", returner="", timeout=5):
    """
    Return the full data about the publication, this is invoked in the same
    way as the publish function

    CLI Example:

    .. code-block:: bash

        salt system.example.com publish.full_data '*' cmd.run 'ls -la /tmp'

    .. admonition:: Attention

        If you need to pass a value to a function argument and that value
        contains an equal sign, you **must** include the argument name.
        For example:

        .. code-block:: bash

            salt '*' publish.full_data test.kwarg arg='cheese=spam'

    """
    return _publish(
        tgt,
        fun,
        arg=arg,
        tgt_type=tgt_type,
        returner=returner,
        timeout=timeout,
        form="full",
        wait=True,
    )


def runner(fun, arg=None, timeout=5):
    """
    Execute a runner on the master and return the data from the runner
    function

    CLI Example:

    .. code-block:: bash

        salt publish.runner manage.down
    """
    arg = _parse_args(arg)

    if "master_uri" not in __opts__:
        return "No access to master. If using salt-call with --local, please remove."
    log.info("Publishing runner '%s' to %s", fun, __opts__["master_uri"])
    auth = salt.crypt.SAuth(__opts__)
    tok = auth.gen_token(b"salt")
    load = {
        "cmd": "minion_runner",
        "fun": fun,
        "arg": arg,
        "tok": tok,
        "tmo": timeout,
        "id": __opts__["id"],
        "no_parse": __opts__.get("no_parse", []),
    }

    with salt.transport.client.ReqChannel.factory(__opts__) as channel:
        try:
            return channel.send(load)
        except SaltReqTimeoutError:
            return "'{}' runner publish timed out".format(fun)



import os

import salt.config
import salt.loader
import salt.roster.ansible as ansible
from tests.support import mixins
from tests.support.mock import patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

EXPECTED = {
    "host1": {
        "host": "host1",
        "passwd": "test123",
        "minion_opts": {
            "escape_pods": 2,
            "halon_system_timeout": 30,
            "self_destruct_countdown": 60,
            "some_server": "foo.southeast.example.com",
        },
    },
    "host2": {
        "host": "host2",
        "passwd": "test123",
        "minion_opts": {
            "escape_pods": 2,
            "halon_system_timeout": 30,
            "self_destruct_countdown": 60,
            "some_server": "foo.southeast.example.com",
        },
    },
    "host3": {
        "host": "host3",
        "passwd": "test123",
        "minion_opts": {
            "escape_pods": 2,
            "halon_system_timeout": 30,
            "self_destruct_countdown": 60,
            "some_server": "foo.southeast.example.com",
        },
    },
}


@skipIf(
    not salt.utils.path.which("ansible-inventory"),
    "Skipping because ansible-inventory is not available",
)
class AnsibleRosterTestCase(TestCase, mixins.LoaderModuleMockMixin):
    @classmethod
    def setUpClass(cls):
        cls.roster_dir = os.path.join(
            RUNTIME_VARS.TESTS_DIR, "unit/files/rosters/ansible/"
        )
        cls.opts = {"roster_defaults": {"passwd": "test123"}}

    @classmethod
    def tearDownClass(cls):
        delattr(cls, "roster_dir")
        delattr(cls, "opts")

    def setup_loader_modules(self):
        opts = salt.config.master_config(
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "master")
        )
        utils = salt.loader.utils(opts, whitelist=["json", "stringutils"])
        runner = salt.loader.runner(opts, utils=utils, whitelist=["salt"])
        return {ansible: {"__utils__": utils, "__opts__": {}, "__runner__": runner}}

    def test_ini(self):
        self.opts["roster_file"] = os.path.join(self.roster_dir, "roster.ini")
        with patch.dict(ansible.__opts__, self.opts):
            ret = ansible.targets("*")
            assert ret == EXPECTED

    def test_yml(self):
        self.opts["roster_file"] = os.path.join(self.roster_dir, "roster.yml")
        with patch.dict(ansible.__opts__, self.opts):
            ret = ansible.targets("*")
            assert ret == EXPECTED

    def test_script(self):
        self.opts["roster_file"] = os.path.join(self.roster_dir, "roster.py")
        with patch.dict(ansible.__opts__, self.opts):
            ret = ansible.targets("*")
            assert ret == EXPECTED



"""
GoGrid Cloud Module
====================

The GoGrid cloud module. This module interfaces with the gogrid public cloud
service. To use Salt Cloud with GoGrid log into the GoGrid web interface and
create an api key. Do this by clicking on "My Account" and then going to the
API Keys tab.

Set up the cloud configuration at ``/etc/salt/cloud.providers`` or
``/etc/salt/cloud.providers.d/gogrid.conf``:

.. code-block:: yaml

    my-gogrid-config:
      # The generated api key to use
      apikey: asdff7896asdh789
      # The apikey's shared secret
      sharedsecret: saltybacon
      driver: gogrid

.. note::

    A Note about using Map files with GoGrid:

    Due to limitations in the GoGrid API, instances cannot be provisioned in parallel
    with the GoGrid driver. Map files will work with GoGrid, but the ``-P``
    argument should not be used on maps referencing GoGrid instances.

.. note::

    A Note about using Map files with GoGrid:

    Due to limitations in the GoGrid API, instances cannot be provisioned in parallel
    with the GoGrid driver. Map files will work with GoGrid, but the ``-P``
    argument should not be used on maps referencing GoGrid instances.

"""

import logging
import pprint
import time

import salt.config as config
import salt.utils.cloud
import salt.utils.hashutils
from salt.exceptions import SaltCloudException, SaltCloudSystemExit

# Get logging started
log = logging.getLogger(__name__)

__virtualname__ = "gogrid"


# Only load in this module if the GoGrid configurations are in place
def __virtual__():
    """
    Check for GoGrid configs
    """
    if get_configured_provider() is False:
        return False

    return __virtualname__


def _get_active_provider_name():
    try:
        return __active_provider_name__.value()
    except AttributeError:
        return __active_provider_name__


def get_configured_provider():
    """
    Return the first configured instance.
    """
    return config.is_provider_configured(
        __opts__,
        _get_active_provider_name() or __virtualname__,
        ("apikey", "sharedsecret"),
    )


def create(vm_):
    """
    Create a single VM from a data dict
    """
    try:
        # Check for required profile parameters before sending any API calls.
        if (
            vm_["profile"]
            and config.is_profile_configured(
                __opts__,
                _get_active_provider_name() or "gogrid",
                vm_["profile"],
                vm_=vm_,
            )
            is False
        ):
            return False
    except AttributeError:
        pass

    __utils__["cloud.fire_event"](
        "event",
        "starting create",
        "salt/cloud/{}/creating".format(vm_["name"]),
        args=__utils__["cloud.filter_event"](
            "creating", vm_, ["name", "profile", "provider", "driver"]
        ),
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    if len(vm_["name"]) > 20:
        raise SaltCloudException("VM names must not be longer than 20 characters")

    log.info("Creating Cloud VM %s", vm_["name"])
    image_id = avail_images()[vm_["image"]]["id"]
    if "assign_public_ip" in vm_:
        host_ip = vm_["assign_public_ip"]
    else:
        public_ips = list_public_ips()
        if not public_ips:
            raise SaltCloudException("No more IPs available")
        host_ip = next(iter(public_ips))

    create_kwargs = {
        "name": vm_["name"],
        "image": image_id,
        "ram": vm_["size"],
        "ip": host_ip,
    }

    __utils__["cloud.fire_event"](
        "event",
        "requesting instance",
        "salt/cloud/{}/requesting".format(vm_["name"]),
        args={
            "kwargs": __utils__["cloud.filter_event"](
                "requesting", create_kwargs, list(create_kwargs)
            ),
        },
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    try:
        data = _query("grid", "server/add", args=create_kwargs)
    except Exception:  # pylint: disable=broad-except
        log.error(
            "Error creating %s on GOGRID\n\n"
            "The following exception was thrown when trying to "
            "run the initial deployment:\n",
            vm_["name"],
            # Show the traceback if the debug logging level is enabled
            exc_info_on_loglevel=logging.DEBUG,
        )
        return False

    ssh_username = config.get_cloud_config_value(
        "ssh_username", vm_, __opts__, default="root"
    )

    def wait_for_apipass():
        """
        Wait for the password to become available, via the API
        """
        try:
            passwords = list_passwords()
            return passwords[vm_["name"]][0]["password"]
        except KeyError:
            pass
        time.sleep(5)
        return False

    vm_["password"] = salt.utils.cloud.wait_for_fun(
        wait_for_apipass,
        timeout=config.get_cloud_config_value(
            "wait_for_fun_timeout", vm_, __opts__, default=15 * 60
        ),
    )

    vm_["ssh_host"] = host_ip
    ret = __utils__["cloud.bootstrap"](vm_, __opts__)
    ret.update(data)

    log.info("Created Cloud VM '%s'", vm_["name"])
    log.debug("'%s' VM creation details:\n%s", vm_["name"], pprint.pformat(data))

    __utils__["cloud.fire_event"](
        "event",
        "created instance",
        "salt/cloud/{}/created".format(vm_["name"]),
        args=__utils__["cloud.filter_event"](
            "created", vm_, ["name", "profile", "provider", "driver"]
        ),
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    return ret


def list_nodes(full=False, call=None):
    """
    List of nodes, keeping only a brief listing

    CLI Example:

    .. code-block:: bash

        salt-cloud -Q
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The list_nodes function must be called with -f or --function."
        )

    ret = {}
    nodes = list_nodes_full("function")
    if full:
        return nodes

    for node in nodes:
        ret[node] = {}
        for item in ("id", "image", "size", "public_ips", "private_ips", "state"):
            ret[node][item] = nodes[node][item]

    return ret


def list_nodes_full(call=None):
    """
    List nodes, with all available information

    CLI Example:

    .. code-block:: bash

        salt-cloud -F
    """
    response = _query("grid", "server/list")

    ret = {}
    for item in response["list"]:
        name = item["name"]
        ret[name] = item

        ret[name]["image_info"] = item["image"]
        ret[name]["image"] = item["image"]["friendlyName"]
        ret[name]["size"] = item["ram"]["name"]
        ret[name]["public_ips"] = [item["ip"]["ip"]]
        ret[name]["private_ips"] = []
        ret[name]["state_info"] = item["state"]
        if "active" in item["state"]["description"]:
            ret[name]["state"] = "RUNNING"

    return ret


def list_nodes_select(call=None):
    """
    Return a list of the VMs that are on the provider, with select fields

    CLI Example:

    .. code-block:: bash

        salt-cloud -S
    """
    return salt.utils.cloud.list_nodes_select(
        list_nodes_full("function"),
        __opts__["query.selection"],
        call,
    )


def avail_locations():
    """
    Available locations
    """
    response = list_common_lookups(kwargs={"lookup": "ip.datacenter"})

    ret = {}
    for item in response["list"]:
        name = item["name"]
        ret[name] = item

    return ret


def avail_sizes():
    """
    Available sizes
    """
    response = list_common_lookups(kwargs={"lookup": "server.ram"})

    ret = {}
    for item in response["list"]:
        name = item["name"]
        ret[name] = item

    return ret


def avail_images():
    """
    Available images
    """
    response = _query("grid", "image/list")

    ret = {}
    for item in response["list"]:
        name = item["friendlyName"]
        ret[name] = item

    return ret


def list_passwords(kwargs=None, call=None):
    """
    List all password on the account

    .. versionadded:: 2015.8.0
    """
    response = _query("support", "password/list")

    ret = {}
    for item in response["list"]:
        if "server" in item:
            server = item["server"]["name"]
            if server not in ret:
                ret[server] = []
            ret[server].append(item)

    return ret


def list_public_ips(kwargs=None, call=None):
    """
    List all available public IPs.

    CLI Example:

    .. code-block:: bash

        salt-cloud -f list_public_ips <provider>

    To list unavailable (assigned) IPs, use:

    CLI Example:

    .. code-block:: bash

        salt-cloud -f list_public_ips <provider> state=assigned

    .. versionadded:: 2015.8.0
    """
    if kwargs is None:
        kwargs = {}

    args = {}
    if "state" in kwargs:
        if kwargs["state"] == "assigned":
            args["ip.state"] = "Assigned"
        else:
            args["ip.state"] = "Unassigned"
    else:
        args["ip.state"] = "Unassigned"

    args["ip.type"] = "Public"

    response = _query("grid", "ip/list", args=args)

    ret = {}
    for item in response["list"]:
        name = item["ip"]
        ret[name] = item

    return ret


def list_common_lookups(kwargs=None, call=None):
    """
    List common lookups for a particular type of item

    .. versionadded:: 2015.8.0
    """
    if kwargs is None:
        kwargs = {}

    args = {}
    if "lookup" in kwargs:
        args["lookup"] = kwargs["lookup"]

    response = _query("common", "lookup/list", args=args)

    return response


def destroy(name, call=None):
    """
    Destroy a machine by name

    CLI Example:

    .. code-block:: bash

        salt-cloud -d vm_name
    """
    if call == "function":
        raise SaltCloudSystemExit(
            "The destroy action must be called with -d, --destroy, -a or --action."
        )

    __utils__["cloud.fire_event"](
        "event",
        "destroying instance",
        "salt/cloud/{}/destroying".format(name),
        args={"name": name},
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    response = _query("grid", "server/delete", args={"name": name})

    __utils__["cloud.fire_event"](
        "event",
        "destroyed instance",
        "salt/cloud/{}/destroyed".format(name),
        args={"name": name},
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    if __opts__.get("update_cachedir", False) is True:
        __utils__["cloud.delete_minion_cachedir"](
            name, _get_active_provider_name().split(":")[0], __opts__
        )

    return response


def reboot(name, call=None):
    """
    Reboot a machine by name

    CLI Example:

    .. code-block:: bash

        salt-cloud -a reboot vm_name

    .. versionadded:: 2015.8.0
    """
    return _query("grid", "server/power", args={"name": name, "power": "restart"})


def stop(name, call=None):
    """
    Stop a machine by name

    CLI Example:

    .. code-block:: bash

        salt-cloud -a stop vm_name

    .. versionadded:: 2015.8.0
    """
    return _query("grid", "server/power", args={"name": name, "power": "stop"})


def start(name, call=None):
    """
    Start a machine by name

    CLI Example:

    .. code-block:: bash

        salt-cloud -a start vm_name

    .. versionadded:: 2015.8.0
    """
    return _query("grid", "server/power", args={"name": name, "power": "start"})


def show_instance(name, call=None):
    """
    Start a machine by name

    CLI Example:

    .. code-block:: bash

        salt-cloud -a show_instance vm_name

    .. versionadded:: 2015.8.0
    """
    response = _query("grid", "server/get", args={"name": name})
    ret = {}
    for item in response["list"]:
        name = item["name"]
        ret[name] = item

        ret[name]["image_info"] = item["image"]
        ret[name]["image"] = item["image"]["friendlyName"]
        ret[name]["size"] = item["ram"]["name"]
        ret[name]["public_ips"] = [item["ip"]["ip"]]
        ret[name]["private_ips"] = []
        ret[name]["state_info"] = item["state"]
        if "active" in item["state"]["description"]:
            ret[name]["state"] = "RUNNING"
    return ret


def _query(
    action=None, command=None, args=None, method="GET", header_dict=None, data=None
):
    """
    Make a web call to GoGrid

    .. versionadded:: 2015.8.0
    """
    vm_ = get_configured_provider()
    apikey = config.get_cloud_config_value("apikey", vm_, __opts__, search_global=False)
    sharedsecret = config.get_cloud_config_value(
        "sharedsecret", vm_, __opts__, search_global=False
    )

    path = "https://api.gogrid.com/api/"

    if action:
        path += action

    if command:
        path += "/{}".format(command)

    log.debug("GoGrid URL: %s", path)

    if not isinstance(args, dict):
        args = {}

    epoch = str(int(time.time()))
    hashtext = "".join((apikey, sharedsecret, epoch))
    args["sig"] = salt.utils.hashutils.md5_digest(hashtext)
    args["format"] = "json"
    args["v"] = "1.0"
    args["api_key"] = apikey

    if header_dict is None:
        header_dict = {}

    if method != "POST":
        header_dict["Accept"] = "application/json"

    decode = True
    if method == "DELETE":
        decode = False

    return_content = None
    result = salt.utils.http.query(
        path,
        method,
        params=args,
        data=data,
        header_dict=header_dict,
        decode=decode,
        decode_type="json",
        text=True,
        status=True,
        opts=__opts__,
    )
    log.debug("GoGrid Response Status Code: %s", result["status"])

    return result["dict"]



import salt.utils.kickstart as kickstart
from tests.support.unit import TestCase


class KickstartTestCase(TestCase):
    def test_clean_args(self):
        ret = kickstart.clean_args({"foo": "bar", "baz": False})
        assert ret == {"foo": "bar"}, ret



import logging
import os
from datetime import datetime

import pytest
import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.plist as plistserializer
import salt.serializers.python as pythonserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
from tests.support.mock import MagicMock, call, patch

try:
    from dateutil.relativedelta import relativedelta

    HAS_DATEUTIL = True
except ImportError:
    HAS_DATEUTIL = False

NO_DATEUTIL_REASON = "python-dateutil is not installed"


log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "python.serialize": pythonserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "plist.serialize": plistserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


@pytest.mark.skipif(not HAS_DATEUTIL, reason=NO_DATEUTIL_REASON)
@pytest.mark.slow_test
def test_retention_schedule():
    """
    Test to execute the retention_schedule logic.

    This test takes advantage of knowing which files it is generating,
    which means it can easily generate list of which files it should keep.
    """

    def generate_fake_files(
        format="example_name_%Y%m%dT%H%M%S.tar.bz2",
        starting=datetime(2016, 2, 8, 9),
        every=relativedelta(minutes=30),
        ending=datetime(2015, 12, 25),
        maxfiles=None,
    ):
        """
        For starting, make sure that it's over a week from the beginning of the month
        For every, pick only one of minutes, hours, days, weeks, months or years
        For ending, the further away it is from starting, the slower the tests run
        Full coverage requires over a year of separation, but that's painfully slow.
        """

        if every.years:
            ts = datetime(starting.year, 1, 1)
        elif every.months:
            ts = datetime(starting.year, starting.month, 1)
        elif every.days:
            ts = datetime(starting.year, starting.month, starting.day)
        elif every.hours:
            ts = datetime(starting.year, starting.month, starting.day, starting.hour)
        elif every.minutes:
            ts = datetime(starting.year, starting.month, starting.day, starting.hour, 0)
        else:
            raise NotImplementedError("not sure what you're trying to do here")

        fake_files = []
        count = 0
        while ending < ts:
            fake_files.append(ts.strftime(format=format))
            count += 1
            if maxfiles and maxfiles == "all" or maxfiles and count >= maxfiles:
                break
            ts -= every
        return fake_files

    fake_name = "/some/dir/name"
    fake_retain = {
        "most_recent": 2,
        "first_of_hour": 4,
        "first_of_day": 7,
        "first_of_week": 6,
        "first_of_month": 6,
        "first_of_year": "all",
    }
    fake_strptime_format = "example_name_%Y%m%dT%H%M%S.tar.bz2"
    fake_matching_file_list = generate_fake_files()
    # Add some files which do not match fake_strptime_format
    fake_no_match_file_list = generate_fake_files(
        format="no_match_%Y%m%dT%H%M%S.tar.bz2", every=relativedelta(days=1)
    )

    def lstat_side_effect(path):
        import re
        from time import mktime

        x = re.match(r"^[^\d]*(\d{8}T\d{6})\.tar\.bz2$", path).group(1)
        ts = mktime(datetime.strptime(x, "%Y%m%dT%H%M%S").timetuple())
        return {
            "st_atime": 0.0,
            "st_ctime": 0.0,
            "st_gid": 0,
            "st_mode": 33188,
            "st_mtime": ts,
            "st_nlink": 1,
            "st_size": 0,
            "st_uid": 0,
        }

    mock_t = MagicMock(return_value=True)
    mock_f = MagicMock(return_value=False)
    mock_lstat = MagicMock(side_effect=lstat_side_effect)
    mock_remove = MagicMock()

    def run_checks(isdir=mock_t, strptime_format=None, test=False):
        expected_ret = {
            "name": fake_name,
            "changes": {"retained": [], "deleted": [], "ignored": []},
            "result": True,
            "comment": "Name provided to file.retention must be a directory",
        }
        if strptime_format:
            fake_file_list = sorted(fake_matching_file_list + fake_no_match_file_list)
        else:
            fake_file_list = sorted(fake_matching_file_list)
        mock_readdir = MagicMock(return_value=fake_file_list)

        with patch.dict(filestate.__opts__, {"test": test}):
            with patch.object(os.path, "isdir", isdir):
                mock_readdir.reset_mock()
                with patch.dict(filestate.__salt__, {"file.readdir": mock_readdir}):
                    with patch.dict(filestate.__salt__, {"file.lstat": mock_lstat}):
                        mock_remove.reset_mock()
                        with patch.dict(
                            filestate.__salt__, {"file.remove": mock_remove}
                        ):
                            if strptime_format:
                                actual_ret = filestate.retention_schedule(
                                    fake_name,
                                    fake_retain,
                                    strptime_format=fake_strptime_format,
                                )
                            else:
                                actual_ret = filestate.retention_schedule(
                                    fake_name, fake_retain
                                )

        if not isdir():
            mock_readdir.assert_has_calls([])
            expected_ret["result"] = False
        else:
            mock_readdir.assert_called_once_with(fake_name)
            ignored_files = fake_no_match_file_list if strptime_format else []
            retained_files = set(
                generate_fake_files(maxfiles=fake_retain["most_recent"])
            )
            junk_list = [
                ("first_of_hour", relativedelta(hours=1)),
                ("first_of_day", relativedelta(days=1)),
                ("first_of_week", relativedelta(weeks=1)),
                ("first_of_month", relativedelta(months=1)),
                ("first_of_year", relativedelta(years=1)),
            ]
            for retainable, retain_interval in junk_list:
                new_retains = set(
                    generate_fake_files(
                        maxfiles=fake_retain[retainable], every=retain_interval
                    )
                )
                # if we generate less than the number of files expected,
                # then the oldest file will also be retained
                # (correctly, since its the first in it's category)
                if (
                    fake_retain[retainable] == "all"
                    or len(new_retains) < fake_retain[retainable]
                ):
                    new_retains.add(fake_file_list[0])
                retained_files |= new_retains

            deleted_files = sorted(
                list(set(fake_file_list) - retained_files - set(ignored_files)),
                reverse=True,
            )
            retained_files = sorted(list(retained_files), reverse=True)
            expected_ret["changes"] = {
                "retained": retained_files,
                "deleted": deleted_files,
                "ignored": ignored_files,
            }
            if test:
                expected_ret["result"] = None
                expected_ret[
                    "comment"
                ] = "{} backups would have been removed from {}.\n" "".format(
                    len(deleted_files), fake_name
                )
            else:
                expected_ret[
                    "comment"
                ] = "{} backups were removed from {}.\n" "".format(
                    len(deleted_files), fake_name
                )
                mock_remove.assert_has_calls(
                    [call(os.path.join(fake_name, x)) for x in deleted_files],
                    any_order=True,
                )

        assert actual_ret == expected_ret

    run_checks(isdir=mock_f)
    run_checks()
    run_checks(test=True)
    run_checks(strptime_format=fake_strptime_format)
    run_checks(strptime_format=fake_strptime_format, test=True)



"""
    :codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

    Tests functions in salt.utils.vsan
"""


import logging

from salt.exceptions import (
    VMwareApiError,
    VMwareObjectRetrievalError,
    VMwareRuntimeError,
)
from salt.utils import vsan
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, PropertyMock, patch
from tests.support.unit import TestCase, skipIf

try:
    from pyVmomi import vim, vmodl  # pylint: disable=no-name-in-module

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False
HAS_PYVSAN = vsan.HAS_PYVSAN


log = logging.getLogger(__name__)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class VsanSupportedTestCase(TestCase):
    """Tests for salt.utils.vsan.vsan_supported"""

    def test_supported_api_version(self):
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(return_value="6.0")
        self.assertTrue(vsan.vsan_supported(mock_si))

    def test_unsupported_api_version(self):
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(return_value="5.0")
        self.assertFalse(vsan.vsan_supported(mock_si))

    def test_api_version_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = "Fake privilege"
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_api_version_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = "VimFault msg"
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(excinfo.exception.strerror, "VimFault msg")

    def test_api_version_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = "RuntimeFault msg"
        mock_si = MagicMock(content=MagicMock(about=MagicMock()))
        type(mock_si.content.about).apiVersion = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.vsan_supported(mock_si)
        self.assertEqual(excinfo.exception.strerror, "RuntimeFault msg")


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class GetVsanClusterConfigSystemTestCase(TestCase, LoaderModuleMockMixin):
    """Tests for salt.utils.vsan.get_vsan_cluster_config_system"""

    def setup_loader_modules(self):
        return {vsan: {"sys": MagicMock(), "ssl": MagicMock()}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_ret = MagicMock()
        patches = (
            (
                "salt.utils.vsan.vsanapiutils.GetVsanVcMos",
                MagicMock(return_value={"vsan-cluster-config-system": self.mock_ret}),
            ),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 9))
        self.mock_context = MagicMock()
        self.mock_create_default_context = MagicMock(return_value=self.mock_context)
        vsan.ssl.create_default_context = self.mock_create_default_context

    def tearDown(self):
        for attr in (
            "mock_si",
            "mock_ret",
            "mock_context",
            "mock_create_default_context",
        ):
            delattr(self, attr)

    def test_ssl_default_context_loaded(self):
        vsan.get_vsan_cluster_config_system(self.mock_si)
        self.mock_create_default_context.assert_called_once_with()
        self.assertFalse(self.mock_context.check_hostname)
        self.assertEqual(self.mock_context.verify_mode, vsan.ssl.CERT_NONE)

    def test_ssl_default_context_not_loaded(self):
        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 8))
        vsan.get_vsan_cluster_config_system(self.mock_si)
        self.assertEqual(self.mock_create_default_context.call_count, 0)

    def test_GetVsanVcMos_call(self):
        mock_get_vsan_vc_mos = MagicMock()
        with patch("salt.utils.vsan.vsanapiutils.GetVsanVcMos", mock_get_vsan_vc_mos):
            vsan.get_vsan_cluster_config_system(self.mock_si)
        mock_get_vsan_vc_mos.assert_called_once_with(
            self.mock_si._stub, context=self.mock_context
        )

    def test_return(self):
        ret = vsan.get_vsan_cluster_config_system(self.mock_si)
        self.assertEqual(ret, self.mock_ret)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'pyvsan' bindings are missing")
class GetVsanDiskManagementSystemTestCase(TestCase, LoaderModuleMockMixin):
    """Tests for salt.utils.vsan.get_vsan_disk_management_system"""

    def setup_loader_modules(self):
        return {vsan: {"sys": MagicMock(), "ssl": MagicMock()}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_ret = MagicMock()
        patches = (
            (
                "salt.utils.vsan.vsanapiutils.GetVsanVcMos",
                MagicMock(return_value={"vsan-disk-management-system": self.mock_ret}),
            ),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 9))
        self.mock_context = MagicMock()
        self.mock_create_default_context = MagicMock(return_value=self.mock_context)
        vsan.ssl.create_default_context = self.mock_create_default_context

    def tearDown(self):
        for attr in (
            "mock_si",
            "mock_ret",
            "mock_context",
            "mock_create_default_context",
        ):
            delattr(self, attr)

    def test_ssl_default_context_loaded(self):
        vsan.get_vsan_disk_management_system(self.mock_si)
        self.mock_create_default_context.assert_called_once_with()
        self.assertFalse(self.mock_context.check_hostname)
        self.assertEqual(self.mock_context.verify_mode, vsan.ssl.CERT_NONE)

    def test_ssl_default_context_not_loaded(self):
        type(vsan.sys).version_info = PropertyMock(return_value=(2, 7, 8))
        vsan.get_vsan_disk_management_system(self.mock_si)
        self.assertEqual(self.mock_create_default_context.call_count, 0)

    def test_GetVsanVcMos_call(self):
        mock_get_vsan_vc_mos = MagicMock()
        with patch("salt.utils.vsan.vsanapiutils.GetVsanVcMos", mock_get_vsan_vc_mos):
            vsan.get_vsan_disk_management_system(self.mock_si)
        mock_get_vsan_vc_mos.assert_called_once_with(
            self.mock_si._stub, context=self.mock_context
        )

    def test_return(self):
        ret = vsan.get_vsan_disk_management_system(self.mock_si)
        self.assertEqual(ret, self.mock_ret)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class GetHostVsanSystemTestCase(TestCase):
    """Tests for salt.utils.vsan.get_host_vsan_system"""

    def setUp(self):
        self.mock_host_ref = MagicMock()
        self.mock_si = MagicMock()
        self.mock_traversal_spec = MagicMock()
        self.mock_vsan_system = MagicMock()
        patches = (
            (
                "salt.utils.vmware.get_managed_object_name",
                MagicMock(return_value="fake_hostname"),
            ),
            (
                "salt.utils.vsan.vmodl.query.PropertyCollector.TraversalSpec",
                MagicMock(return_value=self.mock_traversal_spec),
            ),
            (
                "salt.utils.vmware.get_mors_with_properties",
                MagicMock(return_value=self.mock_traversal_spec),
            ),
            (
                "salt.utils.vmware.get_mors_with_properties",
                MagicMock(return_value=[{"object": self.mock_vsan_system}]),
            ),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_get_managed_object_name.assert_called_once_with(self.mock_host_ref)

    def test_hostname_argument(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name",
            MagicMock(return_value="fake_hostname"),
        ):
            vsan.get_host_vsan_system(
                self.mock_si, self.mock_host_ref, hostname="passedin_hostname"
            )
        self.assertEqual(mock_get_managed_object_name.call_count, 0)

    def test_traversal_spec(self):
        mock_traversal_spec = MagicMock(return_value=self.mock_traversal_spec)
        with patch(
            "salt.utils.vmware.vmodl.query.PropertyCollector.TraversalSpec",
            mock_traversal_spec,
        ):

            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_traversal_spec.assert_called_once_with(
            path="configManager.vsanSystem", type=vim.HostSystem, skip=False
        )

    def test_get_mors_with_properties(self):
        mock_get_mors = MagicMock(return_value=[{"object": self.mock_vsan_system}])
        with patch("salt.utils.vmware.get_mors_with_properties", mock_get_mors):
            vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        mock_get_mors.assert_called_once_with(
            self.mock_si,
            vim.HostVsanSystem,
            property_list=["config.enabled"],
            container_ref=self.mock_host_ref,
            traversal_spec=self.mock_traversal_spec,
        )

    def test_empty_mors_result(self):
        mock_get_mors = MagicMock(return_value=None)
        with patch("salt.utils.vmware.get_mors_with_properties", mock_get_mors):

            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        self.assertEqual(
            excinfo.exception.strerror,
            "Host's 'fake_hostname' VSAN system was not retrieved",
        )

    def test_valid_mors_result(self):
        res = vsan.get_host_vsan_system(self.mock_si, self.mock_host_ref)
        self.assertEqual(res, self.mock_vsan_system)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class CreateDiskgroupTestCase(TestCase):
    """Tests for salt.utils.vsan.create_diskgroup"""

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_initialise_disk_mapping = MagicMock(return_value=self.mock_task)
        self.mock_vsan_disk_mgmt_system = MagicMock(
            InitializeDiskMappings=self.mock_initialise_disk_mapping
        )
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            (
                "salt.utils.vmware.get_managed_object_name",
                MagicMock(return_value="fake_hostname"),
            ),
            (
                "salt.utils.vsan.vim.VimVsanHostDiskMappingCreationSpec",
                MagicMock(return_value=self.mock_spec),
            ),
            ("salt.utils.vsan._wait_for_tasks", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock_get_managed_object_name.assert_called_once_with(self.mock_host_ref)

    def test_vsan_spec_all_flash(self):
        self.mock_cap_disk1.ssd = True
        vsan.create_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_cache_disk,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertEqual(
            self.mock_spec.capacityDisks, [self.mock_cap_disk1, self.mock_cap_disk2]
        )
        self.assertEqual(self.mock_spec.cacheDisks, [self.mock_cache_disk])
        self.assertEqual(self.mock_spec.creationType, "allFlash")
        self.assertEqual(self.mock_spec.host, self.mock_host_ref)

    def test_vsan_spec_hybrid(self):
        self.mock_cap_disk1.ssd = False
        vsan.create_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_cache_disk,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_cap_disk1.ssd = False
        self.assertEqual(self.mock_spec.creationType, "hybrid")

    def test_initialize_disk_mapping(self):
        vsan.create_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_cache_disk,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_initialise_disk_mapping.assert_called_once_with(self.mock_spec)

    def test_initialize_disk_mapping_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = "Fake privilege"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_initialize_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = "vim_fault"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "vim_fault")

    def test_initialize_disk_mapping_raise_method_not_found(self):
        err = vmodl.fault.MethodNotFound()
        err.method = "fake_method"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "Method 'fake_method' not found")

    def test_initialize_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = "runtime_fault"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "runtime_fault")

    def test__wait_for_tasks(self):
        mock___wait_for_tasks = MagicMock()
        with patch("salt.utils.vsan._wait_for_tasks", mock___wait_for_tasks):
            vsan.create_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_cache_disk,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock___wait_for_tasks.assert_called_once_with([self.mock_task], self.mock_si)

    def test_result(self):
        res = vsan.create_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_cache_disk,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertTrue(res)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class AddCapacityToDiskGroupTestCase(TestCase):
    """Tests for salt.utils.vsan.add_capacity_to_diskgroup"""

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_initialise_disk_mapping = MagicMock(return_value=self.mock_task)
        self.mock_vsan_disk_mgmt_system = MagicMock(
            InitializeDiskMappings=self.mock_initialise_disk_mapping
        )
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            (
                "salt.utils.vmware.get_managed_object_name",
                MagicMock(return_value="fake_hostname"),
            ),
            (
                "salt.utils.vsan.vim.VimVsanHostDiskMappingCreationSpec",
                MagicMock(return_value=self.mock_spec),
            ),
            ("salt.utils.vsan._wait_for_tasks", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock_get_managed_object_name.assert_called_once_with(self.mock_host_ref)

    def test_vsan_spec_all_flash(self):
        self.mock_cap_disk1.ssd = True
        vsan.add_capacity_to_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertEqual(
            self.mock_spec.capacityDisks, [self.mock_cap_disk1, self.mock_cap_disk2]
        )
        self.assertEqual(self.mock_spec.cacheDisks, [self.mock_cache_disk])
        self.assertEqual(self.mock_spec.creationType, "allFlash")
        self.assertEqual(self.mock_spec.host, self.mock_host_ref)

    def test_vsan_spec_hybrid(self):
        self.mock_cap_disk1.ssd = False
        vsan.add_capacity_to_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_cap_disk1.ssd = False
        self.assertEqual(self.mock_spec.creationType, "hybrid")

    def test_initialize_disk_mapping(self):
        vsan.add_capacity_to_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_initialise_disk_mapping.assert_called_once_with(self.mock_spec)

    def test_initialize_disk_mapping_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = "Fake privilege"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_initialize_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = "vim_fault"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "vim_fault")

    def test_initialize_disk_mapping_raise_method_not_found(self):
        err = vmodl.fault.MethodNotFound()
        err.method = "fake_method"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "Method 'fake_method' not found")

    def test_initialize_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = "runtime_fault"
        self.mock_vsan_disk_mgmt_system.InitializeDiskMappings = MagicMock(
            side_effect=err
        )
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "runtime_fault")

    def test__wait_for_tasks(self):
        mock___wait_for_tasks = MagicMock()
        with patch("salt.utils.vsan._wait_for_tasks", mock___wait_for_tasks):
            vsan.add_capacity_to_diskgroup(
                self.mock_si,
                self.mock_vsan_disk_mgmt_system,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock___wait_for_tasks.assert_called_once_with([self.mock_task], self.mock_si)

    def test_result(self):
        res = vsan.add_capacity_to_diskgroup(
            self.mock_si,
            self.mock_vsan_disk_mgmt_system,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertTrue(res)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class RemoveCapacityFromDiskGroup(TestCase):
    """Tests for salt.utils.vsan.remove_capacity_from_diskgroup"""

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_remove_disk = MagicMock(return_value=self.mock_task)
        self.mock_host_vsan_system = MagicMock(RemoveDisk_Task=self.mock_remove_disk)
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            (
                "salt.utils.vmware.get_managed_object_name",
                MagicMock(return_value="fake_hostname"),
            ),
            (
                "salt.utils.vsan.get_host_vsan_system",
                MagicMock(return_value=self.mock_host_vsan_system),
            ),
            (
                "salt.utils.vsan.vim.HostMaintenanceSpec",
                MagicMock(return_value=self.mock_spec),
            ),
            ("salt.utils.vsan.vim.VsanHostDecommissionMode", MagicMock()),
            ("salt.utils.vmware.wait_for_task", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.remove_capacity_from_diskgroup(
                self.mock_si,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock_get_managed_object_name.assert_called_once_with(self.mock_host_ref)

    def test_maintenance_mode_evacuate_all_data(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertEqual(
            self.mock_spec.vsanMode.objectAction,
            vim.VsanHostDecommissionModeObjectAction.evacuateAllData,
        )

    def test_maintenance_mode_no_action(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
            data_evacuation=False,
        )
        self.assertEqual(
            self.mock_spec.vsanMode.objectAction,
            vim.VsanHostDecommissionModeObjectAction.noAction,
        )

    def test_remove_disk(self):
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_remove_disk.assert_called_once_with(
            disk=[self.mock_cap_disk1, self.mock_cap_disk2],
            maintenanceSpec=self.mock_spec,
        )

    def test_remove_disk_raise_no_permission(self):
        err = vim.fault.NoPermission()
        err.privilegeId = "Fake privilege"
        self.mock_host_vsan_system.RemoveDisk_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_remove_disk_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = "vim_fault"
        self.mock_host_vsan_system.RemoveDisk_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "vim_fault")

    def test_remove_disk_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = "runtime_fault"
        self.mock_host_vsan_system.RemoveDisk_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.remove_capacity_from_diskgroup(
                self.mock_si,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        self.assertEqual(excinfo.exception.strerror, "runtime_fault")

    def test_wait_for_tasks(self):
        mock_wait_for_task = MagicMock()
        with patch("salt.utils.vmware.wait_for_task", mock_wait_for_task):
            vsan.remove_capacity_from_diskgroup(
                self.mock_si,
                self.mock_host_ref,
                self.mock_diskgroup,
                [self.mock_cap_disk1, self.mock_cap_disk2],
            )
        mock_wait_for_task.assert_called_once_with(
            self.mock_task, "fake_hostname", "remove_capacity"
        )

    def test_result(self):
        res = vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertTrue(res)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class RemoveDiskgroup(TestCase):
    """Tests for salt.utils.vsan.remove_diskgroup"""

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_remove_disk_mapping = MagicMock(return_value=self.mock_task)
        self.mock_host_vsan_system = MagicMock(
            RemoveDiskMapping_Task=self.mock_remove_disk_mapping
        )
        self.mock_host_ref = MagicMock()
        self.mock_cache_disk = MagicMock()
        self.mock_diskgroup = MagicMock(ssd=self.mock_cache_disk)
        self.mock_cap_disk1 = MagicMock()
        self.mock_cap_disk2 = MagicMock()
        self.mock_spec = MagicMock()
        patches = (
            (
                "salt.utils.vmware.get_managed_object_name",
                MagicMock(return_value="fake_hostname"),
            ),
            (
                "salt.utils.vsan.get_host_vsan_system",
                MagicMock(return_value=self.mock_host_vsan_system),
            ),
            (
                "salt.utils.vsan.vim.HostMaintenanceSpec",
                MagicMock(return_value=self.mock_spec),
            ),
            ("salt.utils.vsan.vim.VsanHostDecommissionMode", MagicMock()),
            ("salt.utils.vmware.wait_for_task", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_get_hostname(self):
        mock_get_managed_object_name = MagicMock(return_value="fake_hostname")
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        mock_get_managed_object_name.assert_called_once_with(self.mock_host_ref)

    def test_maintenance_mode_evacuate_all_data(self):
        vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.assertEqual(
            self.mock_spec.vsanMode.objectAction,
            vim.VsanHostDecommissionModeObjectAction.evacuateAllData,
        )

    def test_maintenance_mode_no_action(self):
        vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
            data_evacuation=False,
        )
        self.assertEqual(
            self.mock_spec.vsanMode.objectAction,
            vim.VsanHostDecommissionModeObjectAction.noAction,
        )

    def test_remove_disk_mapping(self):
        vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        vsan.remove_capacity_from_diskgroup(
            self.mock_si,
            self.mock_host_ref,
            self.mock_diskgroup,
            [self.mock_cap_disk1, self.mock_cap_disk2],
        )
        self.mock_remove_disk_mapping.assert_called_once_with(
            mapping=[self.mock_diskgroup], maintenanceSpec=self.mock_spec
        )

    def test_remove_disk_mapping_raise_no_permission(self):
        vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        err = vim.fault.NoPermission()
        err.privilegeId = "Fake privilege"
        self.mock_host_vsan_system.RemoveDiskMapping_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_remove_disk_mapping_raise_vim_fault(self):
        err = vim.fault.VimFault()
        err.msg = "vim_fault"
        self.mock_host_vsan_system.RemoveDiskMapping_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareApiError) as excinfo:
            vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(excinfo.exception.strerror, "vim_fault")

    def test_remove_disk_mapping_raise_runtime_fault(self):
        err = vmodl.RuntimeFault()
        err.msg = "runtime_fault"
        self.mock_host_vsan_system.RemoveDiskMapping_Task = MagicMock(side_effect=err)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        self.assertEqual(excinfo.exception.strerror, "runtime_fault")

    def test_wait_for_tasks(self):
        mock_wait_for_task = MagicMock()
        with patch("salt.utils.vmware.wait_for_task", mock_wait_for_task):
            vsan.remove_diskgroup(self.mock_si, self.mock_host_ref, self.mock_diskgroup)
        mock_wait_for_task.assert_called_once_with(
            self.mock_task, "fake_hostname", "remove_diskgroup"
        )

    def test_result(self):
        res = vsan.remove_diskgroup(
            self.mock_si, self.mock_host_ref, self.mock_diskgroup
        )
        self.assertTrue(res)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class GetClusterVsanInfoTestCase(TestCase, LoaderModuleMockMixin):
    """Tests for salt.utils.vsan.get_cluster_vsan_info"""

    def setup_loader_modules(self):
        return {vsan: {}}

    def setUp(self):
        self.mock_cl_ref = MagicMock()
        self.mock_si = MagicMock()
        patches = (
            ("salt.utils.vmware.get_managed_object_name", MagicMock()),
            (
                "salt.utils.vmware.get_service_instance_from_managed_object",
                MagicMock(return_value=self.mock_si),
            ),
            ("salt.utils.vsan.get_vsan_cluster_config_system", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ("mock_si", "mock_cl_ref"):
            delattr(self, attr)

    def test_get_managed_object_name_call(self):
        mock_get_managed_object_name = MagicMock()
        with patch(
            "salt.utils.vmware.get_managed_object_name", mock_get_managed_object_name
        ):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_get_managed_object_name.assert_called_once_with(self.mock_cl_ref)

    def test_get_vsan_cluster_config_system_call(self):
        mock_get_vsan_cl_syst = MagicMock()
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system", mock_get_vsan_cl_syst
        ):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_get_vsan_cl_syst.assert_called_once_with(self.mock_si)

    def test_VsanClusterGetConfig_call(self):
        mock_vsan_sys = MagicMock()
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(return_value=mock_vsan_sys),
        ):
            vsan.get_cluster_vsan_info(self.mock_cl_ref)
        mock_vsan_sys.VsanClusterGetConfig.assert_called_once_with(self.mock_cl_ref)

    def test_VsanClusterGetConfig_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = "Fake privilege"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterGetConfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_VsanClusterGetConfig_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = "VimFault msg"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterGetConfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(excinfo.exception.strerror, "VimFault msg")

    def test_VsanClusterGetConfig_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = "RuntimeFault msg"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterGetConfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan.get_cluster_vsan_info(self.mock_cl_ref)
        self.assertEqual(excinfo.exception.strerror, "RuntimeFault msg")


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class ReconfigureClusterVsanTestCase(TestCase):
    """Tests for salt.utils.vsan.reconfigure_cluster_vsan"""

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_task = MagicMock()
        self.mock_cl_reconf = MagicMock(return_value=self.mock_task)
        self.mock_get_vsan_conf_sys = MagicMock(
            return_value=MagicMock(VsanClusterReconfig=self.mock_cl_reconf)
        )
        self.mock_cl_ref = MagicMock()
        self.mock_cl_vsan_spec = MagicMock()
        patches = (
            ("salt.utils.vmware.get_managed_object_name", MagicMock()),
            (
                "salt.utils.vmware.get_service_instance_from_managed_object",
                MagicMock(return_value=self.mock_si),
            ),
            (
                "salt.utils.vsan.get_vsan_cluster_config_system",
                self.mock_get_vsan_conf_sys,
            ),
            ("salt.utils.vsan._wait_for_tasks", MagicMock()),
        )
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in (
            "mock_si",
            "mock_cl_reconf",
            "mock_get_vsan_conf_sys",
            "mock_cl_ref",
            "mock_cl_vsan_spec",
            "mock_task",
        ):
            delattr(self, attr)

    def test_get_cluster_name_call(self):
        get_managed_object_name_mock = MagicMock()
        with patch(
            "salt.utils.vmware.get_managed_object_name", get_managed_object_name_mock
        ):

            vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        get_managed_object_name_mock.assert_called_once_with(self.mock_cl_ref)

    def test_get_service_instance_call(self):
        get_service_instance_from_managed_object_mock = MagicMock()
        with patch(
            "salt.utils.vmware.get_service_instance_from_managed_object",
            get_service_instance_from_managed_object_mock,
        ):

            vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        get_service_instance_from_managed_object_mock.assert_called_once_with(
            self.mock_cl_ref
        )

    def test_get_vsan_cluster_config_system_call(self):
        vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        self.mock_get_vsan_conf_sys.assert_called_once_with(self.mock_si)

    def test_cluster_reconfig_call(self):
        vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        self.mock_cl_reconf.assert_called_once_with(
            self.mock_cl_ref, self.mock_cl_vsan_spec
        )

    def test_cluster_reconfig_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = "Fake privilege"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterReconfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_cluster_reconfig_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = "VimFault msg"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterReconfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        self.assertEqual(excinfo.exception.strerror, "VimFault msg")

    def test_cluster_reconfig_raises_vmodl_runtime_error(self):
        exc = vmodl.RuntimeFault()
        exc.msg = "VimRuntime msg"
        with patch(
            "salt.utils.vsan.get_vsan_cluster_config_system",
            MagicMock(
                return_value=MagicMock(VsanClusterReconfig=MagicMock(side_effect=exc))
            ),
        ):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        self.assertEqual(excinfo.exception.strerror, "VimRuntime msg")

    def test__wait_for_tasks_call(self):
        mock_wait_for_tasks = MagicMock()
        with patch("salt.utils.vsan._wait_for_tasks", mock_wait_for_tasks):
            vsan.reconfigure_cluster_vsan(self.mock_cl_ref, self.mock_cl_vsan_spec)
        mock_wait_for_tasks.assert_called_once_with([self.mock_task], self.mock_si)


@skipIf(not HAS_PYVMOMI, "The 'pyvmomi' library is missing")
@skipIf(not HAS_PYVSAN, "The 'vsan' ext library is missing")
class _WaitForTasks(TestCase, LoaderModuleMockMixin):
    """Tests for salt.utils.vsan._wait_for_tasks"""

    def setup_loader_modules(self):
        return {vsan: {}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_tasks = MagicMock()
        patches = (("salt.utils.vsan.vsanapiutils.WaitForTasks", MagicMock()),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ("mock_si", "mock_tasks"):
            delattr(self, attr)

    def test_wait_for_tasks_call(self):
        mock_wait_for_tasks = MagicMock()
        with patch("salt.utils.vsan.vsanapiutils.WaitForTasks", mock_wait_for_tasks):
            vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        mock_wait_for_tasks.assert_called_once_with(self.mock_tasks, self.mock_si)

    def test_wait_for_tasks_raises_no_permission(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = "Fake privilege"
        with patch(
            "salt.utils.vsan.vsanapiutils.WaitForTasks", MagicMock(side_effect=exc)
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(
            excinfo.exception.strerror,
            "Not enough permissions. Required privilege: Fake privilege",
        )

    def test_wait_for_tasks_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = "VimFault msg"
        with patch(
            "salt.utils.vsan.vsanapiutils.WaitForTasks", MagicMock(side_effect=exc)
        ):
            with self.assertRaises(VMwareApiError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(excinfo.exception.strerror, "VimFault msg")

    def test_wait_for_tasks_raises_vmodl_runtime_error(self):
        exc = vmodl.RuntimeFault()
        exc.msg = "VimRuntime msg"
        with patch(
            "salt.utils.vsan.vsanapiutils.WaitForTasks", MagicMock(side_effect=exc)
        ):
            with self.assertRaises(VMwareRuntimeError) as excinfo:
                vsan._wait_for_tasks(self.mock_tasks, self.mock_si)
        self.assertEqual(excinfo.exception.strerror, "VimRuntime msg")



import os
import shutil

import pytest
import salt.utils.platform
from tests.support.case import SSHCase
from tests.support.unit import skipIf


@skipIf(salt.utils.platform.is_windows(), "salt-ssh not available on Windows")
class SSHMineTest(SSHCase):
    """
    testing salt-ssh with mine
    """

    @pytest.mark.slow_test
    def test_ssh_mine_get(self):
        """
        test salt-ssh with mine
        """
        ret = self.run_function("mine.get", ["localhost test.arg"], wipe=False)
        self.assertEqual(ret["localhost"]["args"], ["itworked"])

    def tearDown(self):
        """
        make sure to clean up any old ssh directories
        """
        salt_dir = self.run_function("config.get", ["thin_dir"], wipe=False)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest
import salt.states.schedule as schedule
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {schedule: {}}


def test_present():
    """
    Test to ensure a job is present in the schedule.
    """
    name = "job1"

    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    mock_dict = MagicMock(side_effect=[ret, []])
    mock_mod = MagicMock(return_value=ret)
    mock_lst = MagicMock(side_effect=[{name: {}}, {name: {}}, {}, {}])
    with patch.dict(
        schedule.__salt__,
        {
            "schedule.list": mock_lst,
            "schedule.build_schedule_item": mock_dict,
            "schedule.modify": mock_mod,
            "schedule.add": mock_mod,
        },
    ):
        assert schedule.present(name) == ret

        with patch.dict(schedule.__opts__, {"test": False}):
            assert schedule.present(name) == ret

            assert schedule.present(name) == ret

        with patch.dict(schedule.__opts__, {"test": True}):
            ret.update({"result": True})
            assert schedule.present(name) == ret


def test_absent():
    """
    Test to ensure a job is absent from the schedule.
    """
    name = "job1"

    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    mock_mod = MagicMock(return_value=ret)
    mock_lst = MagicMock(side_effect=[{name: {}}, {}])
    with patch.dict(
        schedule.__salt__, {"schedule.list": mock_lst, "schedule.delete": mock_mod}
    ):
        with patch.dict(schedule.__opts__, {"test": False}):
            assert schedule.absent(name) == ret

        with patch.dict(schedule.__opts__, {"test": True}):
            comt = "Job job1 not present in schedule"
            ret.update({"comment": comt, "result": True})
            assert schedule.absent(name) == ret



"""
    tests.unit.test_test_module_name
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

import fnmatch
import os

import salt.utils.path
import salt.utils.stringutils
from tests.support.paths import list_test_mods
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase

EXCLUDED_DIRS = [
    os.path.join("tests", "integration", "cloud", "helpers"),
    os.path.join("tests", "integration", "files"),
    os.path.join("tests", "kitchen", "tests"),
    os.path.join("tests", "perf"),
    os.path.join("tests", "pkg"),
    os.path.join("tests", "support"),
    os.path.join("tests", "unit", "files"),
    os.path.join("tests", "unit", "modules", "inspectlib"),
    os.path.join("tests", "unit", "modules", "nxos"),
    os.path.join("tests", "unit", "modules", "zypp"),
    os.path.join("tests", "unit", "setup"),
    os.path.join("tests", "unit", "templates", "files"),
]
INCLUDED_DIRS = [
    os.path.join("tests", "kitchen", "tests", "*", "tests", "*"),
]
EXCLUDED_FILES = [
    os.path.join("tests", "buildpackage.py"),
    os.path.join("tests", "committer_parser.py"),
    os.path.join("tests", "consist.py"),
    os.path.join("tests", "eventlisten.py"),
    os.path.join("tests", "jenkins.py"),
    os.path.join("tests", "minionswarm.py"),
    os.path.join("tests", "modparser.py"),
    os.path.join("tests", "packdump.py"),
    os.path.join("tests", "runtests.py"),
    os.path.join("tests", "salt-tcpdump.py"),
    os.path.join("tests", "saltsh.py"),
    os.path.join("tests", "unit", "test_pytest_pass_fail.py"),
    os.path.join("tests", "unit", "transport", "mixins.py"),
    os.path.join("tests", "unit", "utils", "scheduler", "base.py"),
    os.path.join("tests", "virtualname.py"),
    os.path.join("tests", "wheeltest.py"),
    os.path.join("tests", "zypp_plugin.py"),
]


class BadTestModuleNamesTestCase(TestCase):
    """
    Unit test case for testing bad names for test modules
    """

    maxDiff = None

    def _match_dirs(self, reldir, matchdirs):
        return any(fnmatch.fnmatchcase(reldir, mdir) for mdir in matchdirs)

    def test_module_name(self):
        """
        Make sure all test modules conform to the test_*.py naming scheme
        """
        excluded_dirs, included_dirs = tuple(EXCLUDED_DIRS), tuple(INCLUDED_DIRS)
        tests_dir = os.path.join(RUNTIME_VARS.CODE_DIR, "tests")
        bad_names = []
        for root, _, files in salt.utils.path.os_walk(tests_dir):
            reldir = os.path.relpath(root, RUNTIME_VARS.CODE_DIR)
            if (
                reldir.startswith(excluded_dirs)
                and not self._match_dirs(reldir, included_dirs)
            ) or reldir.endswith("__pycache__"):
                continue
            for fname in files:
                if fname in ("__init__.py", "conftest.py") or not fname.endswith(".py"):
                    continue
                relpath = os.path.join(reldir, fname)
                if relpath in EXCLUDED_FILES:
                    continue
                if not fname.startswith("test_"):
                    bad_names.append(relpath)

        error_msg = "\n\nPlease rename the following files:\n"
        for path in bad_names:
            directory, filename = path.rsplit(os.sep, 1)
            filename, _ = os.path.splitext(filename)
            error_msg += "  {} -> {}/test_{}.py\n".format(
                path, directory, filename.split("_test")[0]
            )

        error_msg += (
            "\nIf you believe one of the entries above should be ignored, please add it to either\n"
            "'EXCLUDED_DIRS' or 'EXCLUDED_FILES' in 'tests/unit/test_module_names.py'.\n"
            "If it is a tests module, then please rename as suggested."
        )
        self.assertEqual([], bad_names, error_msg)

    def test_module_name_source_match(self):
        """
        Check all the test mods and check if they correspond to actual files in
        the codebase. If this test fails, then a test module is likely not
        named correctly, and should be adjusted.

        If a test module doesn't have a natural name match (as does this very
        file), then its should be included in the "ignore" tuple below.
        However, if there is no matching source code file, then you should
        consider mapping it to files manually via tests/filename_map.yml.
        """
        ignore = (
            "integration.cli.test_custom_module",
            "integration.cli.test_grains",
            "integration.client.test_kwarg",
            "integration.client.test_runner",
            "integration.client.test_standard",
            "integration.client.test_syndic",
            "integration.cloud.test_cloud",
            "integration.doc.test_man",
            "integration.externalapi.test_venafiapi",
            "integration.grains.test_custom",
            "integration.loader.test_ext_grains",
            "integration.loader.test_ext_modules",
            "integration.logging.handlers.test_logstash_mod",
            "integration.logging.test_jid_logging",
            "integration.master.test_clear_funcs",
            "integration.master.test_event_return",
            "integration.minion.test_executor",
            "integration.minion.test_minion_cache",
            "integration.minion.test_timeout",
            "integration.modules.test_decorators",
            "integration.modules.test_pkg",
            "integration.modules.test_service",
            "integration.modules.test_state_jinja_filters",
            "integration.modules.test_sysctl",
            "integration.netapi.rest_cherrypy.test_app_pam",
            "integration.netapi.rest_tornado.test_app",
            "integration.netapi.test_client",
            "integration.output.test_output",
            "integration.pillar.test_pillar_include",
            "integration.proxy.test_shell",
            "integration.proxy.test_simple",
            "integration.reactor.test_reactor",
            "integration.returners.test_noop_return",
            "integration.runners.test_runner_returns",
            "integration.shell.test_arguments",
            "integration.shell.test_auth",
            "integration.shell.test_call",
            "integration.shell.test_cloud",
            "integration.shell.test_cp",
            "integration.shell.test_enabled",
            "integration.shell.test_key",
            "integration.shell.test_master",
            "integration.shell.test_master_tops",
            "integration.shell.test_minion",
            "integration.shell.test_proxy",
            "integration.shell.test_runner",
            "integration.shell.test_saltcli",
            "integration.shell.test_spm",
            "integration.shell.test_syndic",
            "integration.spm.test_build",
            "integration.spm.test_files",
            "integration.spm.test_info",
            "integration.spm.test_install",
            "integration.spm.test_remove",
            "integration.spm.test_repo",
            "integration.ssh.test_deploy",
            "integration.ssh.test_grains",
            "integration.ssh.test_jinja_filters",
            "integration.ssh.test_master",
            "integration.ssh.test_mine",
            "integration.ssh.test_pillar",
            "integration.ssh.test_pre_flight",
            "integration.ssh.test_raw",
            "integration.ssh.test_saltcheck",
            "integration.ssh.test_state",
            "integration.states.test_compiler",
            "integration.states.test_handle_error",
            "integration.states.test_handle_iorder",
            "integration.states.test_match",
            "integration.states.test_renderers",
            "integration.wheel.test_client",
            "unit.cache.test_cache",
            "unit.serializers.test_serializers",
            "unit.setup.test_install",
            "unit.setup.test_man",
            "unit.states.test_postgres",
            "unit.test_doc",
            "unit.test_mock",
            "unit.test_module_names",
            "unit.test_proxy_minion",
            "unit.test_pytest_pass_fail",
            "unit.test_simple",
            "unit.test_virtualname",
            "unit.test_zypp_plugins",
            "unit.utils.scheduler.test_error",
            "unit.utils.scheduler.test_eval",
            "unit.utils.scheduler.test_helpers",
            "unit.utils.scheduler.test_maxrunning",
            "unit.utils.scheduler.test_postpone",
            "unit.utils.scheduler.test_run_job",
            "unit.utils.scheduler.test_schedule",
            "unit.utils.scheduler.test_skip",
            "unit.auth.test_auth",
        )
        errors = []

        def _format_errors(errors):
            msg = (
                "The following {} test module(s) could not be matched to a "
                "source code file:\n\n".format(len(errors))
            )
            msg += "".join(errors)
            return msg

        for mod_name in list_test_mods():
            if mod_name in ignore:
                # Test module is being ignored, skip it
                continue

            # Separate the test_foo away from the rest of the mod name, because
            # we'll need to remove the "test_" from the beginning and add .py
            stem, flower = mod_name.rsplit(".", 1)
            # Lop off the integration/unit from the beginning of the mod name
            try:
                stem = stem.split(".", 1)[1]
            except IndexError:
                # This test mod was in the root of the unit/integration dir
                stem = ""

            # The path from the root of the repo
            relpath = salt.utils.path.join(
                stem.replace(".", os.sep), ".".join((flower[5:], "py"))
            )

            # The full path to the file we expect to find
            abspath = salt.utils.path.join(RUNTIME_VARS.SALT_CODE_DIR, relpath)

            if not os.path.isfile(abspath):
                # Maybe this is in a dunder init?
                alt_relpath = salt.utils.path.join(relpath[:-3], "__init__.py")
                alt_abspath = salt.utils.path.join(abspath[:-3], "__init__.py")
                if os.path.isfile(alt_abspath):
                    # Yep, it is. Carry on!
                    continue

                errors.append("{} (expected: {})\n".format(mod_name, relpath))

        assert not errors, _format_errors(errors)
